"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sign_rule_1 = __importDefault(require("./rules/sign-rule"));
const memoize_1 = __importDefault(require("../../../core/utils/memoize"));
const is_string_1 = __importDefault(require("../../../core/utils/is-string"));
const abbreviation_rule_1 = __importDefault(require("./rules/abbreviation-rule"));
const decimal_places_rule_1 = __importDefault(require("./rules/decimal-places-rule"));
const string_includes_1 = __importDefault(require("../../../core/utils/string-includes"));
const optional_decimal_places_rule_1 = __importDefault(require("./rules/optional-decimal-places-rule"));
const pattern_regexp_utils_1 = require("../../utils/pattern-regexp-utils");
/**
 * <i> The regExp tests for:
 *     - 0.00##X
 *     - 0,0.00##X
 *     - #.00##X (without leading zeros)
 *     - #,#.00##X (without leading zeros)
 */
const numberPositionRule = (patternMask) => {
    const numberPartRegExp = /((((0|#)+,)?(0|#)+(\.([0#X]|\[0+\])+)?){1})/;
    return pattern_regexp_utils_1.patternReplace(patternMask, numberPartRegExp, `'ɵn'`);
};
/**
 * Minimum leading integer digits rule
 * This will define the minimum amount of digits on the integer part (left-most grouped zeroes).
 *     - (12.34, '0000.0') =>  '0012.3'
 *     - (12.34, '0000,0.0')  =>  '0,012.3'
 *     - (0.34, '#.0') => '.3'
 *     - (1.34, '#.0') => '1.3'
 *
 * <i> It always pick the left-most amount of zeros, so:
 *     - If pattern has NO thousands separator ('000.0'), then the amount at the left of the DOT is used.
 *     - If pattern HAS thousands separator ('00,0.0'), then the amount at the left of the COMMA is used.
 * <i> This will remove the integer zero for numbers between 1 and -1 (e.g. 0.23 or -0.5)
 *     - If pattern integer part is option (0.24, '#.00') => '.24'
 */
const minimumIntegerDigitsRule = (patternMask) => {
    const patternMaskWithoutEscapedText = pattern_regexp_utils_1.patternRemoveEscapedText(patternMask);
    const patternMaskIntegerPart = patternMaskWithoutEscapedText.split('.')[0].split(',')[0];
    // If it has '#' in the integer part, sets the minimumIntegerDigits to 0
    if (/#/g.test(patternMaskIntegerPart)) {
        return 0;
    }
    return (patternMaskIntegerPart.match(/0/g) || []).length;
};
// If sign is not included, put sign at the left of the number
const addSignPositionIfItDoesNotExists = (patternMask) => {
    if (string_includes_1.default(patternMask, `'ɵs'`) || string_includes_1.default(patternMask, `'ɵnps'`))
        return patternMask;
    return patternMask.replace(`'ɵn'`, _ => `'ɵs''ɵn'`);
};
const baseParsePattern = (inputPattern) => {
    const resolvedInputPattern = is_string_1.default(inputPattern) && inputPattern || '0,0.##########';
    const [patternMaskAfterSignRule, signRules] = sign_rule_1.default(resolvedInputPattern);
    const [patternMaskAfterAbbreviationRule, abbreviationRules] = abbreviation_rule_1.default(patternMaskAfterSignRule);
    const [patternMaskAfterOptionalDecimalPlacesRule, optionalDecimalPlacesRules] = optional_decimal_places_rule_1.default(patternMaskAfterAbbreviationRule);
    const outputPatternMask = patternMaskAfterOptionalDecimalPlacesRule;
    const outputPatternMaskWithoutEscapedText = pattern_regexp_utils_1.patternRemoveEscapedText(outputPatternMask);
    const decimalPlacesRules = decimal_places_rule_1.default(outputPatternMask);
    const minimumIntegerDigits = minimumIntegerDigitsRule(outputPatternMask);
    const grouping = outputPatternMaskWithoutEscapedText.indexOf(',') > -1;
    const patternMaskAfterHandlingNumberPosition = numberPositionRule(outputPatternMask);
    const patternMaskWithEnsuredSignPosition = addSignPositionIfItDoesNotExists(patternMaskAfterHandlingNumberPosition);
    const patternMask = patternMaskWithEnsuredSignPosition;
    return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, signRules), abbreviationRules), optionalDecimalPlacesRules), decimalPlacesRules), { grouping,
        minimumIntegerDigits,
        patternMask });
};
const parsePattern = memoize_1.default(baseParsePattern);
exports.default = parsePattern;
//# sourceMappingURL=parse-pattern.js.map